<?php


/**
 * Created by JetBrains PhpStorm.
 * User: jorgefonseca
 * Date: 27/03/12
 * Time: 14:55
 * To change this template use File | Settings | File Templates.
 */
class LoginLogicProdutos
{

    private $_nmorganizacao;
    private $_nrorg;
    private $_cdfilial;
    private $_email;
    private $_senha;
    private $_cdoperador;
    /** @var $_operador OperadorEntity */
    private $_operador; // OperadorEntity
    /** @var $_filial FilialEntity */
    private $_filial;   // FilialEntity
    /** @var $_perfilacesso PerfilacessoEntity */
    private $_perfilacesso;   // PerfilacessoEntity
    /** @var $_personal PersonalLogic */
    private $_personal;


    public function __construct($nmorganizacao, $cdfilial, $acesso, $senha) {

        $this->_nmorganizacao = $nmorganizacao;
        $this->_cdfilial = $cdfilial;
        $this->_email = $acesso;
        $this->_senha = $senha;

        $this->_validaLogin();
    }


    /**
     *  Valida as informacoes recebidas no login.
     *
     * @return array
     */
    private function _validaLogin() {
//        $this->_validaOrganizacao();
//        $this->_validaFilial();
        $this->_validaAcesso();
//        $this->_validaFilialOperador();

//        $this->_validaPerfilacesso();
    }


    private function _validaOrganizacao() {
        $organizacao = Organizacao::retornaOrganizacaoPeloNomeUpcase($this->_nmorganizacao);
        if (!$organizacao) {
            throw new Exception('Organização inválida!');
        }
        $this->_nrorg = $organizacao->getNrorg();
    }


    /**
 * Valida se filial existe.
 *
 * @throws LoginException
 * @return void
 */
    private function _validaFilial(){
        $filial = Filial::consultaFilial($this->_cdfilial, $this->_nrorg);
        if (!$filial) {
            throw new LoginException('msg_valida_filial'); // Filial Nao Cadastrada.
        }
        $this->_filial = $filial;
    }


    /**
    * Valida operador que acessa.
    *
    * @throws LoginException
    */
    private function _validaAcesso() {

        $this->_validaOperador();
        // Se operador tem senha para o sistema Delphi, sera essa a senha considerada.
        if ($this->_operador->getCdsenhoper()) {
            $operador = $this->_validaOperadorWebService();
            if (is_object($operador) !== true) {
                throw new LoginException($operador);
            }
        } else if ($this->_validaAlteraSenha() === false) {
            throw new LoginException('msg_valida_senha_operador'); // Senha Invalida.
        }

    }




    /**
     * Valida se operador esta cadastrado na filial.
     *
     * @throws LoginException
     * @return void
     */
    private function _validaFilialOperador(){
        $filioper = Filioper::consultaFilioperFilial($this->_cdfilial, $this->_operador->getCdoperador(), $this->_nrorg);
        if (!$filioper) {
            throw new LoginException('msg_valida_operador_filial'); // Operador nao cadastrado para a filial.
        }
    }


    /**
     *  Valida ou altera senha.
     *
     * @return void|boolean
     */
    private function _validaAlteraSenha(){
        $criptosenha = md5($this->_senha);
        $cdsenhaoperweb = ($this->_senha !== '')?$this->_operador->getCdsenhaoperweb() : null;

        if (($cdsenhaoperweb === null || $cdsenhaoperweb === 'd41d8cd98f00b204e9800998ecf8427e') && $this->_senha !== '') {
            Operador::alteraSenhaOperador($this->_operador->getCdoperador(), $criptosenha);
            return null;
        } else if ($cdsenhaoperweb !== $criptosenha) {
            return false;
        }
        return true;
    }


    /**
     * Valida se operador existe ou esta inativo.
     * @throws LoginException
     */
    private function _validaOperador() {

        $email = "/[A-Za-z0-9]+([_.-][A-Za-z0-9]+)*@[A-Za-z0-9]+([_.-][A-Za-z0-9]+)*\\.[A-Za-z0-9]{2,4}/";

        if (preg_match($email, $this->_email)) {
            $operador = Operador::consultaOperadorPorEmail($this->_email);
        } else {
            throw new \LoginException('msg_email_invalido');
        }

        if (!is_object($operador)) {
            throw new LoginException('msg_operador_n_cadastrado'); // Operador Nao Cadastrado.
        }

        if ($operador->getIdoperativo() !== 'A'){
            throw new LoginException('msg_operador_inativo'); // Operador encontra-se inativo.
        }

        $this->_cdoperador = $operador->getCdoperador();
        $this->_operador = $operador;
    }


    /**
     * Verificacao caso nao exista perfilacessoSelecionado.
     *
     * @return PerfilacessoEntity
     * @throws LoginException
     * @throws Exception
     */
    private function _validaPerfilacesso()
    {

        /** @var $ParamgeralEntity ParamgeralEntity */
        $ParamgeralEntity = ParamgeralEntity::retornaParamgeral($this->_nrorg);
        if ($ParamgeralEntity->getIdutctrlaces() == 'N') {
            return true;
        }

        /** @var $perfacessparc PerfacessparcEntity */
        $perfacessparc =
            PerfacessparcEntity::retornaPerfacessparc(
                $this->_operador->getNrparcnegocio(),
                $this->_nrorg);

        if (!is_object($perfacessparc[0])) {
            throw new LoginException('msg_naoExiste_perfilAcesso');
            return false;
        }
        $perfacessparc = reset($perfacessparc);
        $this->verificaValidadeAcesso($perfacessparc);
        $perfilacesso = PerfilacessoEntity::retornaPerfilacesso($perfacessparc->getNrperfilacesso(), $this->_nrorg);
        if (!is_object($perfilacesso)) {
            throw new LoginException('msg_perfil_acesso_operador'); // Acesso nao parametrizado para o operador.
            return false;
        }
        $this->_perfilacesso = $perfilacesso;
        return true;
    }


    /**
     * Verificacao caso nao exista perfilacessoSelecionado.
     *
     * @param $perfil
     *
     * @return PerfilacessoEntity
     */
    private function existePerfilacessoSelecionado($perfil)
    {
        /** @var $perfacessparc PerfacessparcEntity */
        $perfacessparc = PerfacessparcEntity::retornaPerfacessparcPorNrperfilacesso(
            $this->_operador->getNrparcnegocio(),
            $perfil,
            $this->_nrorg
        );
        $this->verificaValidadeAcesso($perfacessparc);
        $perfilacesso = PerfilacessoEntity::retornaPerfilacesso($perfil, $this->_nrorg);
        tk_warning(gettype($perfilacesso).'oi');
        return $perfilacesso;
    }


    /**
     * Verifica validade de acesso.
     *
     * @param PerfacessparcEntity $perfacessparc
     * @throws LoginException
     * @throws Exception
     */
    private function verificaValidadeAcesso(PerfacessparcEntity $perfacessparc)
    {
        if ($perfacessparc->getIdativo() === 'N') {
            throw new LoginException('msg_perfilAcesso_inativo');
        }
        if ($perfacessparc->getDtvalperfilparc() !== null) {
            $validade = new Zend_Date($perfacessparc->getDtvalperfilparc());
            $hoje = new Zend_Date(date("d/m/Y"));
            if ($hoje->compare($validade) > 0) {
                $perfacessparc->setIdativo('N');
                $perfacessparc->persist();
                throw new LoginException('msg_data_validade_acesso'); // Data de validade de acesso expirada.
            }
        }
    }


    /**
     * @return mixed
     */
    public function getNrorg()
    {
        return $this->_nrorg;
    }


    /**
     * @return OperadorEntity
     */
    public function getOperador()
    {
        return $this->_operador;
    }


    /**
     * @return FilialEntity
     */
    public function getFilial()
    {
        return $this->_filial;
    }


    /**
     * @return PersonalLogic
     */
    public function getPersonal()
    {
        return $this->_personal;
    }


    /**
     * @return string
     */
    public function getCdfilial()
    {
        return $this->_cdfilial;
    }


    /**
     * @return string
     */
    public function getCdoperador()
    {
        return $this->_cdoperador;
    }

    /**
     * @return string
     */
    public function getPerfilAcesso()
    {
        return $this->_perfilacesso;
    }
}
