/******************************************************************************/
/* SITE - Sistemas Integrados TEKNISA                                         */
/******************************************************************************/
/* Nome:    _GPASTPO_CADASTR_AUTO_GPA                                         */
/*                                                                            */
/* Descrio:                                                                 */
/*  REALIZA O CADASTRAMENTO AUTOMATICO DO GRUPO DE BEM,  TIPO DE DEPRECIACAO  */
/*  E REGIME DE PREPRIEDADE NAS TELAS DE ENTRADA DO BEM OU CADASTRO DO BEM.   */
/*                                                                            */
/* Criao:                                                                   */
/* 17/04/2012 - ARTHUR VIANNA                                                 */
/* -------------------------------------------------------------------------- */
/* P_RETORNO = '01' : 'Cadastramento efetuado com sucesso.'                   */
/* P_RETORNO = '02' : 'Alguns bens do intervalo j sofreram depreciao.      */
/*                       Esses bens no foram modificados.'                   */
/* P_RETORNO = '03' : 'Nenhum bem foi modificado. Os bens selecionados        */
/*                       j sofreram depreciao.'                            */
/*                                                                            */
/******************************************************************************/
create or replace PROCEDURE CADASTR_AUTO_GPA( 
    P_IDENTRCAD   IN VARCHAR2, 
    P_EMPRESA     IN EMPRESA.CDEMPRESA%TYPE, 
    P_CHAVE       IN VARCHAR2, 
    P_INICIAL     IN VARCHAR2, 
    P_FINAL       IN VARCHAR2, 
    P_NVGRUPOBEM  IN GRUPOBEM.CDGRUPOBEM%TYPE, 
    P_NVTPDEPREC  IN TIPODEPREC.CDTIPODEPREC%TYPE,
    P_NVREGIMEBEM IN REGIMEBEM.CDREGIMEBEM%TYPE, 
    P_TPAROPROP   IN VARCHAR2 DEFAULT NULL, 
    P_IDENTBEM    IN VARCHAR2 DEFAULT NULL, 
    P_INDUTILIZ   IN VARCHAR2 DEFAULT NULL, 
    P_NRPARCELA   IN VARCHAR2 DEFAULT NULL, 
    P_RETORNO OUT VARCHAR2 ) 
IS 
   V_GRUPOBEM   VARCHAR2(06); 
   V_TPDEPBEM   VARCHAR2(06); 
   V_REGIMEBEM  VARCHAR2(06); 
   V_IDDEPRECIAGRBEM VARCHAR2(01); 
   V_QTMESDEPREGRBEM FLOAT; 
   V_IDDEPRECIAFISC  VARCHAR2(01); 
   V_QTMESDEPREFISC  FLOAT;
   V_NRCOETIPODEP    FLOAT; 
   V_VRTXINCFISTPDEP FLOAT; 
   V_NRCOETIPODEPFIS FLOAT; 
   V_VRTXINCFISTPFIS FLOAT; 
   V_IDDEPREGBEM     VARCHAR2(01); 
   V_IDDEPREGBEMFIS  VARCHAR2(01);
   V_VIDAUTIL FLOAT; 
   V_TAXABASICA FLOAT; 
   V_TAXACONT FLOAT; 
   V_MESCONT  FLOAT; 
   V_TAXAFISC FLOAT; 
   V_MESFISC  FLOAT; 
   V_ALTERACAOBENS    VARCHAR2(01); 
   V_BENSNAOALTERADOS    INTEGER; 
   V_TPAROPROP  VARCHAR2(02); 
   V_IDENTBEM   VARCHAR2(02); 
   V_INDUTILIZ  VARCHAR2(02); 
   V_NRPARCELA  VARCHAR2(02);
   EXISTE_IMP NUMBER;
BEGIN
   IF P_IDENTRCAD = 'E' THEN   
      FOR C_ENTRADABEM IN (SELECT NRSEQITCAPAENT, NMBEMITCAPAENT, CDGRUPOITCENT, CDTIPODEPREC, CDREGIMEBEM, 
                                  IDNOVOUSAITCENT, QTMESUSOITCENT 
                             FROM ITCAPAENTRABEM 
                            WHERE (CDEMPRESA = P_EMPRESA) 
                              AND (NRSEQCAPAENTBEM = P_CHAVE) 
                              AND (NRSEQITCAPAENT BETWEEN P_INICIAL AND P_FINAL)
                            ORDER BY NRSEQITCAPAENT) LOOP 
         IF P_NVGRUPOBEM IS NULL THEN 
            V_GRUPOBEM := C_ENTRADABEM.CDGRUPOITCENT; 
         ELSE 
            V_GRUPOBEM := P_NVGRUPOBEM; 
         END IF; 
         IF P_NVTPDEPREC IS NULL THEN 
            V_TPDEPBEM := C_ENTRADABEM.CDTIPODEPREC;
         ELSE 
            V_TPDEPBEM := P_NVTPDEPREC; 
         END IF; 
         IF P_NVREGIMEBEM IS NULL THEN 
            V_REGIMEBEM := C_ENTRADABEM.CDREGIMEBEM; 
         ELSE 
            V_REGIMEBEM := P_NVREGIMEBEM; 
         END IF;
         SELECT IDDEPRECIAGRBEM, QTMESDEPREGRBEM, IDDEPRECIAFISC, QTMESDEPREFISC 
           INTO V_IDDEPRECIAGRBEM, V_QTMESDEPREGRBEM, V_IDDEPRECIAFISC, V_QTMESDEPREFISC 
           FROM GRUPOBEM 
          WHERE CDEMPRESA = P_EMPRESA 
            AND CDGRUPOBEM = V_GRUPOBEM; 
         SELECT NRCOETIPODEP, VRTXINCFISTPDEP, NRCOETIPODEPFIS, VRTXINCFISTPFIS
           INTO V_NRCOETIPODEP, V_VRTXINCFISTPDEP, V_NRCOETIPODEPFIS, V_VRTXINCFISTPFIS 
           FROM TIPODEPREC 
          WHERE CDEMPRESA = P_EMPRESA 
            AND CDTIPODEPREC = V_TPDEPBEM; 
         SELECT IDDEPREGBEM, IDDEPREGBEMFIS 
           INTO V_IDDEPREGBEM, V_IDDEPREGBEMFIS 
           FROM REGIMEBEM 
          WHERE CDEMPRESA = P_EMPRESA
            AND CDREGIMEBEM = V_REGIMEBEM; 
          IF V_NRCOETIPODEP IS NULL THEN 
             V_NRCOETIPODEP := 1; 
          END IF; 
          IF V_VRTXINCFISTPDEP IS NULL THEN 
             V_VRTXINCFISTPDEP := 0; 
          END IF;
          V_VIDAUTIL :=  V_QTMESDEPREGRBEM; 
          IF V_VIDAUTIL > 0 THEN 
             IF C_ENTRADABEM.IDNOVOUSAITCENT = 'U' THEN 
                IF C_ENTRADABEM.QTMESUSOITCENT >= (V_VIDAUTIL/2) THEN 
                   V_VIDAUTIL := V_VIDAUTIL/2; 
                ELSE 
                   V_VIDAUTIL := V_VIDAUTIL - C_ENTRADABEM.QTMESUSOITCENT; 
                END IF;
             END IF;           
             V_TAXABASICA := (100/V_VIDAUTIL)*12; 
             V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPDEP/100)); 
             V_TAXACONT :=  V_TAXABASICA * V_NRCOETIPODEP; 
             V_MESCONT :=   (100/V_TAXACONT)*12; 
          ELSE 
             V_VIDAUTIL := 1; 
             V_TAXABASICA := (100/V_VIDAUTIL)*12;
             V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPDEP/100)); 
             V_TAXACONT :=  V_TAXABASICA * V_NRCOETIPODEP; 
             V_MESCONT :=   (100/V_TAXACONT)*12; 
          END IF; 
          IF V_NRCOETIPODEPFIS IS NULL THEN 
             V_NRCOETIPODEPFIS := 1; 
          END IF;
          IF V_VRTXINCFISTPFIS IS NULL THEN 
             V_VRTXINCFISTPFIS := 0; 
          END IF; 
          V_VIDAUTIL :=  V_QTMESDEPREFISC; 
          IF V_VIDAUTIL > 0 THEN 
             IF C_ENTRADABEM.IDNOVOUSAITCENT = 'U' THEN 
                IF C_ENTRADABEM.QTMESUSOITCENT >= (V_VIDAUTIL/2) THEN
                   V_VIDAUTIL := V_VIDAUTIL/2; 
                ELSE 
                   V_VIDAUTIL := V_VIDAUTIL - C_ENTRADABEM.QTMESUSOITCENT; 
                END IF; 
             END IF;           
             V_TAXABASICA := (100/V_VIDAUTIL)*12; 
             V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPFIS/100));
             V_TAXAFISC :=  V_TAXABASICA * V_NRCOETIPODEPFIS; 
             IF V_TAXAFISC = 0 THEN 
                V_TAXAFISC := 1; 
             END IF; 
             V_MESFISC :=   (100/V_TAXAFISC)*12; 
          ELSE 
             V_VIDAUTIL := 1;
             V_TAXABASICA := (100/V_VIDAUTIL)*12; 
             V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPFIS/100)); 
             V_TAXAFISC :=  V_TAXABASICA * V_NRCOETIPODEPFIS; 
             IF V_TAXAFISC = 0 THEN 
                V_TAXAFISC := 1; 
             END IF; 
             V_MESFISC :=   (100/V_TAXAFISC)*12;
          END IF; 
         UPDATE ITCAPAENTRABEM 
         SET CDGRUPOITCENT = V_GRUPOBEM, 
             CDTIPODEPREC  = V_TPDEPBEM, 
             CDREGIMEBEM   =  V_REGIMEBEM, 
             QTMESDEPITCENT  =  round(V_MESCONT),
             VRTAXDEPITCENT  =  V_TAXACONT, 
             QTMESDEPITCENFI =   round(V_MESFISC), 
             VRTAXDEPITCENFI =  V_TAXAFISC 
         WHERE (CDEMPRESA = P_EMPRESA) 
           AND (NRSEQCAPAENTBEM = P_CHAVE) 
           AND (NRSEQITCAPAENT = C_ENTRADABEM.NRSEQITCAPAENT); 
      END LOOP;
      P_RETORNO := '03'; 
   END IF; 
   IF P_IDENTRCAD = 'C' THEN
      V_ALTERACAOBENS := 'N'; /* SE ALGUM BEM MODIFICADO, ESTA VARIAVEL DEVE SER ALTERADA, MESMO QUE SEJA SOMENTE UM BEM.
                                CASO ELA CONTINUE COMO "N", A MENSAGEM INFORMANDO Q NENHUM BEM FOI ALTERADO  MOSTRADA AO USURIO */ 
      V_BENSNAOALTERADOS := 0; 
      FOR C_CADASTROBEM IN (SELECT CDBEMATIVO, CDPATRIBEMATIVO, NMBEMATIVO, CDGRUPOBEM, CDTIPODEPREC, CDREGIMEBEM, 
                                   VRJADEPBEMATIVO, VRJADEPBEMFISC, QTMESUSOBEM, IDNOVOUSABEM,
                                   IDTPAPROPPISCOF, IDIDENTBEMPISCOF, IDINDICUTLPISCOF, IDNRPARCPISCOF  
                              FROM BEMATIVO 
                             WHERE (CDEMPRESA = P_EMPRESA)
                               AND (CDPATRIBEMATIVO BETWEEN P_INICIAL AND P_FINAL)
                             ORDER BY CDPATRIBEMATIVO)LOOP
         IF NOT ((C_CADASTROBEM.VRJADEPBEMATIVO > 0) OR (C_CADASTROBEM.VRJADEPBEMFISC > 0)) THEN
            V_ALTERACAOBENS := 'S'; /* SE ALGUM BEM MODIFICADO, ESTA VARIAVEL DEVE SER ALTERADA, MESMO QUE SEJA SOMENTE UM BEM.
                                 CASO ELA CONTINUE COMO "N", A MENSAGEM INFORMANDO Q NENHUM BEM FOI ALTERADO  MOSTRADA AO USURIO */
            IF P_NVGRUPOBEM IS NULL THEN
               V_GRUPOBEM := C_CADASTROBEM.CDGRUPOBEM;
            ELSE
               V_GRUPOBEM := P_NVGRUPOBEM;
            END IF;
            IF P_NVTPDEPREC IS NULL THEN
               V_TPDEPBEM := C_CADASTROBEM.CDTIPODEPREC;
            ELSE
               V_TPDEPBEM := P_NVTPDEPREC;
            END IF;
            IF P_NVREGIMEBEM IS NULL THEN
               V_REGIMEBEM := C_CADASTROBEM.CDREGIMEBEM;
            ELSE
               V_REGIMEBEM := P_NVREGIMEBEM;
            END IF;

          SELECT IDDEPRECIAGRBEM, QTMESDEPREGRBEM, IDDEPRECIAFISC, QTMESDEPREFISC
             INTO V_IDDEPRECIAGRBEM, V_QTMESDEPREGRBEM, V_IDDEPRECIAFISC, V_QTMESDEPREFISC
             FROM GRUPOBEM
            WHERE CDEMPRESA = P_EMPRESA
              AND CDGRUPOBEM = V_GRUPOBEM;

           SELECT NRCOETIPODEP, VRTXINCFISTPDEP, NRCOETIPODEPFIS, VRTXINCFISTPFIS
             INTO V_NRCOETIPODEP, V_VRTXINCFISTPDEP, V_NRCOETIPODEPFIS, V_VRTXINCFISTPFIS
             FROM TIPODEPREC
            WHERE CDEMPRESA = P_EMPRESA
              AND CDTIPODEPREC = V_TPDEPBEM;

           SELECT IDDEPREGBEM, IDDEPREGBEMFIS
             INTO V_IDDEPREGBEM, V_IDDEPREGBEMFIS
             FROM REGIMEBEM
            WHERE CDEMPRESA = P_EMPRESA
              AND CDREGIMEBEM = V_REGIMEBEM;

            IF V_NRCOETIPODEP IS NULL THEN
               V_NRCOETIPODEP := 1;
            END IF;
            IF V_VRTXINCFISTPDEP IS NULL THEN
               V_VRTXINCFISTPDEP := 0;
            END IF;
            V_VIDAUTIL :=  V_QTMESDEPREGRBEM;
            IF V_VIDAUTIL > 0 THEN
               IF C_CADASTROBEM.IDNOVOUSABEM = 'U' THEN
                  IF C_CADASTROBEM.QTMESUSOBEM >= (V_VIDAUTIL/2) THEN
                     V_VIDAUTIL := V_VIDAUTIL/2;
                  ELSE
                     V_VIDAUTIL := V_VIDAUTIL - C_CADASTROBEM.QTMESUSOBEM;
                  END IF;
               END IF;
               V_TAXABASICA := (100/V_VIDAUTIL)*12;
               V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPDEP/100));
               V_TAXACONT :=  V_TAXABASICA * V_NRCOETIPODEP;
               V_MESCONT :=   (100/V_TAXACONT)*12;
            ELSE
               V_VIDAUTIL := 1;
               V_TAXABASICA := (100/V_VIDAUTIL)*12;
               V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPDEP/100));
               V_TAXACONT :=  V_TAXABASICA * V_NRCOETIPODEP;
               V_MESCONT :=   (100/V_TAXACONT)*12;
            END IF;
            IF V_NRCOETIPODEPFIS IS NULL THEN
               V_NRCOETIPODEPFIS := 1;
            END IF;
            IF V_VRTXINCFISTPFIS IS NULL THEN
               V_VRTXINCFISTPFIS := 0;
            END IF;
            V_VIDAUTIL :=  V_QTMESDEPREFISC;
            IF V_VIDAUTIL > 0 THEN
              IF C_CADASTROBEM.IDNOVOUSABEM = 'U' THEN
                 IF C_CADASTROBEM.QTMESUSOBEM >= (V_VIDAUTIL/2) THEN
                    V_VIDAUTIL := V_VIDAUTIL/2;
                 ELSE
                    V_VIDAUTIL := V_VIDAUTIL - C_CADASTROBEM.QTMESUSOBEM;
                 END IF;
              END IF;
               V_TAXABASICA := (100/V_VIDAUTIL)*12;
               V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPFIS/100));
               V_TAXAFISC :=  V_TAXABASICA * V_NRCOETIPODEPFIS;
               IF V_TAXAFISC = 0 THEN
                  V_TAXAFISC := 1;
               END IF;
               V_MESFISC :=   (100/V_TAXAFISC)*12;
            ELSE
               V_VIDAUTIL := 1;
               V_TAXABASICA := (100/V_VIDAUTIL)*12;
               V_TAXABASICA := V_TAXABASICA + (V_TAXABASICA * (V_VRTXINCFISTPFIS/100));
               V_TAXAFISC :=  V_TAXABASICA * V_NRCOETIPODEPFIS;
               IF V_TAXAFISC = 0 THEN
                  V_TAXAFISC := 1;
               END IF;
               V_MESFISC :=   (100/V_TAXAFISC)*12;
            END IF;
            UPDATE BEMATIVO
               SET CDGRUPOBEM       = V_GRUPOBEM,
                   CDTIPODEPREC     = V_TPDEPBEM,
                   CDREGIMEBEM      = V_REGIMEBEM,
                   VRTAXANODEPBEM   = V_TAXACONT,
                   QTMESDEPBEM      = round(V_MESCONT),
                   VRTXANODEPBEMFI  = V_TAXAFISC,
                   QTMESDEPBEMFISC  = round(V_MESFISC),
                   QTMESFALTADBEM   = round(V_MESCONT),
                   QTMESFALTDBEMFIS = round(V_MESFISC)
             WHERE (CDEMPRESA  =  P_EMPRESA)
               AND (CDPATRIBEMATIVO =  C_CADASTROBEM.CDPATRIBEMATIVO);
         ELSE
            V_BENSNAOALTERADOS := V_BENSNAOALTERADOS + 1;
         END IF;

         /* Aqui comea a rotina de  cadastramento dos parmetros de PIS/COFINS.
            OBS: Caso o bem ja tenha impostos cadastrados, os parmetros sero desconsiderados. */
         SELECT COUNT(CDIMPOSTO)
           INTO EXISTE_IMP
           FROM IMPOSTOBEM
          WHERE CDEMPRESA  = P_EMPRESA
            AND CDBEMATIVO = C_CADASTROBEM.CDBEMATIVO;
         IF EXISTE_IMP = 0 THEN
            V_ALTERACAOBENS := 'S'; /* SE ALGUM BEM MODIFICADO, ESTA VARIAVEL DEVE SER ALTERADA, MESMO QUE SEJA SOMENTE UM BEM.
                                       CASO ELA CONTINUE COMO "N", A MENSAGEM INFORMANDO Q NENHUM BEM FOI ALTERADO  MOSTRADA AO USURIO */
            IF P_TPAROPROP IS NULL THEN
               V_TPAROPROP := C_CADASTROBEM.IDTPAPROPPISCOF;
            ELSE
               V_TPAROPROP := P_TPAROPROP;
            END IF;
            IF V_TPAROPROP <> '99' THEN
               IF P_IDENTBEM IS NULL THEN
                  V_IDENTBEM := C_CADASTROBEM.IDIDENTBEMPISCOF;
               ELSE
                  V_IDENTBEM := P_IDENTBEM;
               END IF;
            ELSE
               V_IDENTBEM := '';
            END IF;
            IF V_TPAROPROP <> '99' THEN
               IF P_INDUTILIZ IS NULL THEN
                  V_INDUTILIZ := C_CADASTROBEM.IDINDICUTLPISCOF;
               ELSE
                  V_INDUTILIZ := P_INDUTILIZ;
               END IF;
            ELSE
               V_INDUTILIZ := '';
            END IF;
            IF V_TPAROPROP = '10' THEN
               IF P_NRPARCELA IS NULL THEN
                  V_NRPARCELA := C_CADASTROBEM.IDNRPARCPISCOF;
               ELSE
                  V_NRPARCELA := P_NRPARCELA;
               END IF;
            ELSE
               V_NRPARCELA :=  '';
            END IF;

            UPDATE BEMATIVO
               SET IDTPAPROPPISCOF  = V_TPAROPROP,
                   IDIDENTBEMPISCOF = V_IDENTBEM,
                   IDINDICUTLPISCOF = V_INDUTILIZ,
                   IDNRPARCPISCOF   = V_NRPARCELA
             WHERE (CDEMPRESA  =  P_EMPRESA)
               AND (CDPATRIBEMATIVO =  C_CADASTROBEM.CDPATRIBEMATIVO);
         ELSE
            V_BENSNAOALTERADOS := V_BENSNAOALTERADOS + 1;
         END IF;
      END LOOP;
      IF V_ALTERACAOBENS = 'N' THEN  /* A VARIVEL COMEA COM 'N' E  ALTERADA PARA 'S' QUANDO ALGUM BEM  ALTERADO NA ROTINA */
          P_RETORNO := '03'; /* NENHUM BEM FOI MODIFICADO NA ROTINA DE CADASTRAMENTO AUTOMTICO */
      ELSE
         IF V_BENSNAOALTERADOS = 0 THEN
            P_RETORNO := '01'; /* CADASTRAMENTO AUTOMTICO DEU CERTO PARA TODOS OS BENS FILTRADOS */
         ELSE
            P_RETORNO := '02'; /* ALGUNS BENS NO FORAM ALTERADOS POR TEREM SIDOS DEPRECIADOS OU J TEREM IMPOSTOS CADASTRADOS */
         END IF;
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CADASTR_AUTO_GPA','5.85.xxx'); END;
/
